package software.eskimo.capacitor.sockets;

import android.util.Log;
import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.net.Socket;
import java.security.NoSuchAlgorithmException;
import javax.net.ssl.SSLContext;
import javax.net.ssl.SSLSocketFactory;
import java.io.IOException;
import javax.net.ssl.TrustManager;
import javax.net.ssl.X509TrustManager;
import java.security.cert.X509Certificate;

public class SocketHandler {
    private String id;
    private Socket socket;
    private OutputStream outputStream;
    private BufferedReader inputStream;
    private SocketDelegate delegate;

    public SocketHandler(String id, SocketDelegate delegate) {
        this.id = id;
        this.delegate = delegate;
    }

    public String getId() {
        return id;
    }

    public void connect(final String host, final int port, final boolean useTLS, final boolean acceptInvalidCertificates) {
        new Thread(() -> {
            try {
                delegate.onStateChanged(id, "connecting");
                
                if (useTLS) {
                    SSLContext sslContext = SSLContext.getInstance("TLS");
                    if (acceptInvalidCertificates) {
                        sslContext.init(null, new TrustManager[]{new X509TrustManager() {
                            @Override
                            public void checkClientTrusted(X509Certificate[] chain, String authType) {}

                            @Override
                            public void checkServerTrusted(X509Certificate[] chain, String authType) {}

                            @Override
                            public X509Certificate[] getAcceptedIssuers() { return new X509Certificate[0]; }
                        }}, new java.security.SecureRandom());
                    } else {
                        sslContext.init(null, null, new java.security.SecureRandom());
                    }
                    SSLSocketFactory socketFactory = sslContext.getSocketFactory();
                    socket = socketFactory.createSocket(host, port);
                } else {
                    socket = new Socket(host, port);
                }
                delegate.onStateChanged(id, "connected");
                outputStream = socket.getOutputStream();
                inputStream = new BufferedReader(new InputStreamReader(socket.getInputStream()));
                receive();
            } catch (Exception e) {
                Log.e("SocketHandler", "Connection error: " + e.getMessage(), e);
                delegate.onStateChanged(id, "disconnected");
            }
        }).start();
    }

    public void send(String message) {
        new Thread(() -> {
            try {
                if (outputStream != null) {
                    outputStream.write(message.getBytes());
                    outputStream.flush();
                }
            } catch (IOException e) {
                Log.e("SocketHandler", "Send error: " + e.getMessage(), e);
                delegate.onStateChanged(id, "disconnected");
            }
        }).start();
    }

    public void disconnect() {
        try {
            if (socket != null) {
                socket.close();
            }
            delegate.onStateChanged(id, "disconnected");
        } catch (IOException e) {
            Log.e("SocketHandler", "Disconnect error: " + e.getMessage(), e);
            delegate.onStateChanged(id, "disconnected");
        }
    }

    private void receive() {
        new Thread(() -> {
            try {
                char[] buffer = new char[1024];
                StringBuilder messageBuilder = new StringBuilder();
                int numCharsRead;
                while ((numCharsRead = inputStream.read(buffer)) != -1) {
                    messageBuilder.append(buffer, 0, numCharsRead);
                    String message = messageBuilder.toString();
                    
                    // Check if the message ends with \r\n (or \n, depending on protocol)
                    if (message.endsWith("\r\n")) {
                        Log.d("SocketHandler", "Message received: " + message);
                        delegate.onMessageReceived(id, message);  // Notify with full message including \r\n
                        messageBuilder.setLength(0);  // Clear the buffer for the next message
                    }
                }
            } catch (IOException e) {
                Log.e("SocketHandler", "Receive error: " + e.getMessage(), e);
                delegate.onStateChanged(id, "disconnected");
            }
        }).start();
    }

    public interface SocketDelegate {
        void onStateChanged(String socketId, String state);
        void onMessageReceived(String socketId, String message);
    }
}
