package software.eskimo.capacitor.sockets;

import android.util.Log;
import java.util.ArrayList;
import java.util.List;

public class Sockets implements SocketHandler.SocketDelegate {

    private List<SocketHandler> sockets = new ArrayList<>();
    private SocketsPlugin plugin;  // Reference to SocketsPlugin for notifying JS

    public Sockets(SocketsPlugin plugin) {
        this.plugin = plugin;  // Pass plugin reference for message forwarding
    }

    public SocketHandler create(String id) {
        SocketHandler socket = new SocketHandler(id, this);
        sockets.add(socket);
        return socket;
    }

    public void connect(String id, String host, int port, boolean useTLS, boolean acceptInvalidCertificates) {
        SocketHandler socket = getSocketById(id);
        if (socket != null) {
            socket.connect(host, port, useTLS, acceptInvalidCertificates);
        }
    }

    public void send(String id, String message) {
        SocketHandler socket = getSocketById(id);
        if (socket != null) {
            socket.send(message);
        }
    }

    public void disconnect(String id) {
        SocketHandler socket = getSocketById(id);
        if (socket != null) {
            socket.disconnect();
        }
    }

    private SocketHandler getSocketById(String id) {
        for (SocketHandler socket : sockets) {
            if (socket.getId().equals(id)) {
                return socket;
            }
        }
        return null;
    }

    // Handle state changes (connected, disconnected)
    @Override
    public void onStateChanged(String socketId, String state) {
        Log.d("Sockets", "Socket state changed: " + state);
        // Call notifyStateListeners in SocketsPlugin
        plugin.notifyStateListeners("state", socketId, state);
    }

    // Handle incoming messages
    @Override
    public void onMessageReceived(String socketId, String message) {
        Log.d("Sockets", "Socket message received: " + message);
        // Call notifyMessageListeners in SocketsPlugin
        plugin.notifyMessageListeners(socketId, message);  // Notify JS about the received message
    }
}
