package software.eskimo.capacitor.sockets;

import android.content.Intent;
import android.util.Log;
import com.getcapacitor.JSObject;
import com.getcapacitor.Plugin;
import com.getcapacitor.PluginCall;
import com.getcapacitor.annotation.CapacitorPlugin;
import com.getcapacitor.PluginMethod;

@CapacitorPlugin(name = "Sockets")
public class SocketsPlugin extends Plugin {

    private Sockets implementation = new Sockets(this);  // Pass the plugin reference

    @PluginMethod
    public void create(PluginCall call) {
        String id = call.getString("id", java.util.UUID.randomUUID().toString());
        implementation.create(id);
        call.resolve();
    }

    @PluginMethod
    public void connect(PluginCall call) {
        String id = call.getString("id", "");
        String host = call.getString("host", "");
        int port = call.getInt("port", 0);
        boolean useTLS = call.getBoolean("useTLS", false);
        boolean acceptInvalidCertificates = call.getBoolean("acceptInvalidCertificates", false);

        // Start the foreground service to keep the socket connection alive in the background
        startSocketForegroundService();
        
        implementation.connect(id, host, port, useTLS, acceptInvalidCertificates);
        call.resolve();
    }

    @PluginMethod
    public void send(PluginCall call) {
        String id = call.getString("id", "");
        String message = call.getString("message", "");

        implementation.send(id, message);
        call.resolve();
    }

    @PluginMethod
    public void disconnect(PluginCall call) {
        String id = call.getString("id", "");

        implementation.disconnect(id);
        call.resolve();
    }

    // Helper method for notifying JavaScript listeners about state changes
    public void notifyStateListeners(String event, String id, String state) {
        JSObject data = new JSObject();
        data.put("id", id);
        data.put("state", state);
        notifyListeners(event, data);
    }

    public void notifyMessageListeners(String id, String message) {
        JSObject data = new JSObject();
        data.put("id", id);
        data.put("message", message);
        
        Log.d("SocketsPlugin", "Message to JS: " + data.toString());

        notifyListeners("message", data);
    }

    // Method to start the foreground service
    private void startSocketForegroundService() {
        Intent serviceIntent = new Intent(getContext(), SocketForegroundService.class);
        getContext().startForegroundService(serviceIntent);
    }
}
